import socket


class Ipv4Address(str):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, name: str) -> str:
        socket.inet_aton(name)
        return name



from pydantic_custom_types.active_directory.util import load_local_file


class RealAdUser(str):
    """
    This class requires env var "USERS_FILE".
    Value must be path to a json file containing a list of AD UPN names(str) to
    validate against
    """
    @staticmethod
    def users() -> list[str]:
        return [x.lower() for x in load_local_file("USERS_FILE")]

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, name: str):
        for user in cls.users():
            if user.lower() == name.lower():
                return name
        else:
            raise ValueError(f'User "{name}" does not exist')




from pydantic_custom_types.active_directory.util import load_local_file


class RealAdGroup(str):
    """
    This class requires env var "GROUPS_FILE".
    Value must be path to a json file containing a list of AD groups(str) to
    validate against
    """
    @staticmethod
    def groups() -> list[str]:
        return load_local_file("GROUPS_FILE")

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def find_group_not_case_sensitive(cls, name: str):
        for g in cls.groups():
            if g.lower() == name.lower():
                return g


    @classmethod
    def validate(cls, name: str):
        if name in cls.groups():
            return name
        else:
            hint = cls.find_group_not_case_sensitive(name)
            if hint:
                raise ValueError(f'Group "{name}" does not exist, did you mean "{hint}"?')
            raise ValueError(f'Group "{name}" does not exist.')



from decouple import config, UndefinedValueError
import json


def load_local_file(env_var):
    try:
        usr = config(env_var)
    except UndefinedValueError as e:
        raise UndefinedValueError(f"Env var not set '{env_var}'\npath to json file of list[str] must be specified to use class\n{e}")

    with open(usr, "r") as uf:
        return json.load(uf)



from string import ascii_lowercase


class Hostname(str):
    NUMBERS = "0123456789"
    LEGAL_CHARS = ascii_lowercase + "-" + "_" + NUMBERS

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, name: str) -> str:
        cls.has_valid_characters(name)
        cls.has_to_many_characters(name)
        cls.start_end_with_dot(name)

        return name

    @classmethod
    def has_valid_characters(cls, name: str):
        if not all(char in cls.LEGAL_CHARS for char in name):
            raise ValueError(f'Value "{name}" maybe only contain [-_0-9a-z]')
        return name

    @classmethod
    def has_to_many_characters(cls, name: str):
        max_length = 64
        if len(name) >= max_length:
            raise ValueError(f"Name must be less than {max_length} characters")
        return name

    @classmethod
    def start_end_with_dot(cls, name: str):
        if name.startswith(".") or name.endswith("."):
            raise ValueError("Name cannot start/end with a dot")
        return name

    @classmethod
    def start_end_with_dash(cls, name: str):
        if name.startswith("-") or name.endswith("-"):
            raise ValueError("Name cannot start/end with a dot")
        return name



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydantic_custom_types',
 'pydantic_custom_types.active_directory',
 'pydantic_custom_types.dns',
 'pydantic_custom_types.ip',
 'pydantic_custom_types.kubernetes']

package_data = \
{'': ['*']}

install_requires = \
['build>=0.9.0,<0.10.0', 'python-decouple>=3.6,<4.0']

setup_kwargs = {
    'name': 'pydantic-custom-types',
    'version': '0.2.2',
    'description': 'Custom types for pydantic used in SRE peipelins for validation input vars',
    'long_description': '# pydantic custom types\nA collection of custom types for pydantic  \nhttps://github.com/pydantic/pydantic\n\nUseful for validation input parameters for infrastructure pipelines when building kubernetes apps/clusters\n\n## Requirements\n\n`python >= 3.10`\n\n## Installation\n\n`pip install pydantic-custom-types`\n\n## Examples\n\n```python\nfrom pydantic import BaseModel\nfrom pydantic_custom_types.kubernetes import NamespaceName, SecretName\n\nclass K8sNamespace(BaseModel):\n    # These string types only allow: lowercase/numbers/dash, cannot start with dash/number\n    # NamepaceName has linit at 63 characters\n    # SecretName has linit at 63 characters\n    name: NamespaceName \n    secret_name: SecretName\n\n# will pass\nK8sNamespace(\n    name="my-namespace",\n    secret_name="my-secret"\n)\n\n# will not pass\nK8sNamespace(\n    name="-0mynameSpace",\n    secret_name="0mysecret"\n)\n```',
    'author': 'per lejon',
    'author_email': 'per.lejon@netigate.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
